
#include "dial.h"


int catch[] = { SIGHUP, SIGINT, SIGQUIT, SIGIOT, SIGSEGV, SIGALRM,
				SIGTERM, SIGFPE, 0 } ;
unsigned char modemibuf[BUFSIZE],*modemip=modemibuf,*modemiq=modemibuf;
unsigned char modemobuf[BUFSIZE],*modemop=modemobuf,*modemoq=modemobuf+SNDBUFSIZE;
char *prompts[] = {
	"OOK", "-CONNECT FAX", "CCONNECT", "NNO CARRIER", "EERROR",
	"NNO DIALTONE", "BBUSY", "NNO ANSWER", "E+FCERROR", 0 } ;
enum promptcodes {
	MODEM_BUSY = 'B', MODEM_CONNECT = 'C', MODEM_OK = 'O',
	MODEM_RING = 'R', MODEM_NO = 'N', MODEM_ERROR = 'E' } ;

extern int _command(char *com,int t);
extern char *_strtabmatch ( char **p, char *s );
extern int _modemdata (int t);
extern void _msleep (int t);
extern int _modemflush ();
extern int _modemputcnow(char c);
extern int _modemputs(const char *s);
extern int _modemundrflw ( int t );
extern char *_modemgets( char *s , int n , int t );
extern int _checktermio ( struct termios *t );
extern void _onsig ( int sig );


int open_modem()
{
	int i,err,flags;
	static struct termios t ;

	modem.modem = open(modem.modemdev, O_RDWR | O_NDELAY);
	if(modem.modem < 0) {
		notice("Modem device is busy.",NOTICE_TYPE_OK,"Dismiss","");
		return FALSE;
	}
	foot("opening modem device...");
	if ( ( flags = fcntl( modem.modem, F_GETFL, 0 ) ) < 0 ||
		fcntl( modem.modem, F_SETFL, ( flags & ~O_NDELAY ) ) < 0 ) {
		notice("Modem device fcntl.",NOTICE_TYPE_OK,"Dismiss","");
		return FALSE;
	}
	
	foot("Opened device");
	
	tcdrain(modem.modem);
	tcflush(modem.modem, TCIOFLUSH);
	
	if (tcgetattr(modem.modem,&t)) {
      		notice("tcgetattr failed.",NOTICE_TYPE_OK,"Dismiss","");
		return FALSE;
	}
	 
	if ( _checktermio ( &t ) ) {
		notice("terminal mode corrupted.",NOTICE_TYPE_OK,"Dismiss","");
		return FALSE;
	}
	
	t.c_iflag = IGNBRK | IGNPAR | IXANY ;
	t.c_oflag = 0;
	t.c_cflag = CS8 | CREAD | CLOCAL | HUPCL ;
	t.c_lflag = 0;
	
	t.c_cc[VMIN]  = 1; 
	t.c_cc[VTIME] = 0; 
	
	cfsetospeed ( &t , B19200 ) ; 
	cfsetispeed ( &t , B19200 ) ; 

	tcdrain ( modem.modem );
	tcflow ( modem.modem , TCOON ) ;
	 
  	if(tcsetattr(modem.modem,TCSANOW,&t)) {
  		notice("tcsetattr failed.",NOTICE_TYPE_OK,"Dismiss","");
  		return FALSE;
  	}


  	_checktermio(&t);
  	
  	for ( i=0 ; catch [ i ] ; i++ ) 
    		if ( signal ( catch [ i ] , _onsig ) == SIG_ERR ) {
			notice("can't set signal handler\n",NOTICE_TYPE_OK,"Dismiss","");
			return FALSE;
    		}
    	    	
	foot("Start modem initializing...");
	if(_command("z",-25) != MODEM_OK) {
		notice("Modem initialize failed.",NOTICE_TYPE_OK,"Dismiss","");
		return FALSE;
	} else {
		foot("success at modem initialize.");
		return TRUE;
	}
}

int dial_start(char *telnum)
{
	char buf[512];
	
	if(modem.modem < 0) 
		return FALSE;
	
	if(!modem.toneorpuls) 	strcpy(buf,"DT");
	else 		strcpy(buf,"DP"); 
	strcat(buf,telnum);
	
	foot("Now dialing. Hang on.");
		_command(buf,1200);
	foot("Hotpage");
	
}

int _command(char *com,int t)
{
	char buf[BUFSIZE],*p="";
	
	sleep(3);
retry:
	
	while ( _modemgets(buf,BUFSIZE, t<0 ? 3 : 0 ) ) 
		foot("Unexpeted response");
 	
	if(com) { _modemputs ( "AT" ); _modemputs( com ); _modemputcnow(CR); }
	
  	while ( t && (p = _modemgets( buf , BUFSIZE, t<0 ? -t : t )) ) {
		                		
		if ( ( p = _strtabmatch ( (char**) prompts , buf ) ) ) break ;
		if ( ! strncmp ( buf, "RING", 4 ) ) { _msleep(100) ; goto retry ; } 
	}

  return p ? *p : EOF ;
}


char *_strtabmatch ( char **p, char *s )
{
	while ( *p && strncmp ( *p+1 , s , strlen ( *p+1 ) ) ) p++ ;
	return ( ! *p || **p == '-' ) ? NULL : *p ;
}


int _modemdata ( int t )
{
  int n ;
  fd_set fds ;

  struct timeval timeout ;

  timeout.tv_sec  = t / 10 ; 
  timeout.tv_usec = ( t % 10 ) * 100000 ;

  FD_ZERO ( &fds ) ;
  FD_SET ( modem.modem , &fds ) ;

  n = select ( modem.modem+1 , &fds , 0 , 0 , t<0 ? 0 : &timeout ) ;

  return n ;
}


void _msleep ( int t )
{
	struct timeval timeout ;
	timeout.tv_sec  = t / 1000 ;
	timeout.tv_usec = ( t % 1000 ) * 1000 ;
	if ( select ( 1 , 0 , 0 , 0 , &timeout ) < 0 ) 
		foot("select failed in _msleep");
}
            
int _modemflush ( )
{ 
	int n=0 ;
	unsigned char *p = modemobuf ;
	
	if ( modemop >= modemoq );
	
	while ( p < modemop && ( n = write( modem.modem , p , modemop-p ) ) >= 0 )
		p += n ? n : 0;
	   

	return n >= 0 ? (modemop = modemobuf , 0) : 0;
} 

#define _modemputc(c) ( modemop >= modemoq && _modemflush() == EOF ? EOF : \
		    (unsigned char) ( *modemop++ = (c) ) )

#define _modemobytes() ( modemop - modemobuf ) 


int _modemputcnow ( char c ) 
{ 
	return _modemputc ( c ) < 0 ? EOF : ( _modemflush() ? EOF : c ); 
}

int _modemputs ( const char *s )
{
	int n=0 ;
	while ( s && *s && ( n = _modemputcnow ( *s++ ) ) != EOF ) ;
	return n ;
}

int _modemundrflw ( int t )
{ 
	int n = _modemdata ( t ) ;
	
	if ( n > 0 )
		if ( ( n = read( modem.modem , modemibuf , BUFSIZE ) ) < 0 )
			foot("Error : fax device read.");
	
	modemiq = ( modemip = modemibuf ) + ( n > 0 ? n : 0 ) ;

	return n > 0 ? n : EOF ;
} 


#define _modemgetc(t) ( modemip >= modemiq && _modemundrflw(t) == EOF ? EOF : \
		    *(unsigned char*)modemip++ )

char *_modemgets( char *s , int n , int t )
{
	int c=0, sta=0 ;
	char *p = s ;

	while ( sta < 4 ) {
		if ( ( c = _modemgetc( t ) ) == EOF ) break ;
		c &= 0x7f ;
		switch ( sta ) {
		case 0 :
		case 1 : sta =               ( iscntrl ( c ) ? 1 : (p=s, 2) ) ; break ;
		case 2 : sta = c == CR ? 3 : ( iscntrl ( c ) ? 1 :       2  ) ; break ;
		case 3 : sta = c == LF ? 4 : ( iscntrl ( c ) ? 1 : (p=s, 2) ) ; break ;
		}
		if ( sta == 2 && p < s+n-1 ) *p++ = c ;
	}

	*p = 0 ;
	if ( p >= s+n-1 ) {
	  	foot("modem response overflow");
	}

	return c == EOF ? 0 : s ;
}

int _checktermio ( struct termios *t )
{
	struct termios s ;
	return tcgetattr ( modem.modem , &s ) != 0 ||
		t->c_iflag != s.c_iflag || t->c_oflag != s.c_oflag
		|| t->c_lflag != s.c_lflag || t->c_cflag != s.c_cflag
	 	|| t->c_cc[VMIN] != s.c_cc[VMIN] || t->c_cc[VTIME]!= s.c_cc[VTIME] ;
}

int close_modem ( )
{
	if(_command("z",-25) != MODEM_OK) {
		notice("Modem reset failed.",NOTICE_TYPE_OK,"Dismiss","");
	} else {
		foot("Modem reset was succeeded.");
	}
	
	close(modem.modem);

	foot("Hotpage");
} 

void _onsig ( int sig ) 
{ 
	notice("terminating on signal",NOTICE_TYPE_OK,"Dismiss","");
	close_modem ( ) ;
	xv_destroy_safe(Hot_parent->frame);
	exit(5) ; 
} 
	    	
    

